function optimize_prior(DynareOptions, ModelInfo, DynareResults, BayesInfo, EstimationInfo, pnames)

% This routine computes the mode of the prior density using an optimization algorithm.

% Copyright © 2015-2017 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <https://www.gnu.org/licenses/>.

DynareResults.dr = set_state_space(DynareResults.dr, ModelInfo, DynareOptions);

% Initialize to the prior mean
xparam1 = Prior.p1;

% Pertubation of the initial condition.
look_for_admissible_initial_condition = true; scale = 1.0; iter  = 0;
while look_for_admissible_initial_condition
    xinit = xparam1+scale*randn(size(xparam1));
    if all(xinit>Prior.p3) && all(xinit<Prior.p4)
        ModelInfo = set_all_parameters(xinit, EstimationInfo, ModelInfo);
        [dr, INFO, ModelInfo, DynareResults] = resol(0, ModelInfo, DynareOptions, DynareResults);
        if ~INFO(1)
            look_for_admissible_initial_condition = false;
        end
    else
        if iter==2000
            scale = scale/1.1;
            iter = 0;
        else
            iter = iter+1;
        end
    end
end

% Maximization of the prior density
[xparams, lpd, hessian_mat] = ...
    maximize_prior_density(xinit, pnames, DynareOptions, ModelInfo, Prior, EstimationInfo, DynareResults);

% Display results.
skipline(2)
disp('------------------')
disp('PRIOR OPTIMIZATION')
disp('------------------')
skipline()
for i = 1:length(xparams)
    dprintf('deep parameter %u: %s.', i, get_the_name(i, 0, ModelInfo, EstimationInfo, DynareOptions))
    dprintf('  Initial condition ........ %s.', num2str(xinit(i)))
    dprintf('  Prior mode ............... %s.', num2str(Prior.p5(i)))
    dprintf('  Optimized prior mode ..... %s.', num2str(xparams(i)))
    skipline()
end
skipline()
